@@ -1,53 +1,19 @@
 <?php
-/**
- * php-token-stream
+/*
+ * This file is part of the PHP_TokenStream package.
  *
- * Copyright (c) 2009-2014, Sebastian Bergmann <sebastian@phpunit.de>.
- * All rights reserved.
+ * (c) Sebastian Bergmann <sebastian@phpunit.de>
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- *   * Redistributions of source code must retain the above copyright
- *     notice, this list of conditions and the following disclaimer.
- *
- *   * Redistributions in binary form must reproduce the above copyright
- *     notice, this list of conditions and the following disclaimer in
- *     the documentation and/or other materials provided with the
- *     distribution.
- *
- *   * Neither the name of Sebastian Bergmann nor the names of his
- *     contributors may be used to endorse or promote products derived
- *     from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
- * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- * @package   PHP_TokenStream
- * @author    Sebastian Bergmann <sebastian@phpunit.de>
- * @copyright 2009-2014 Sebastian Bergmann <sebastian@phpunit.de>
- * @license   http://www.opensource.org/licenses/BSD-3-Clause  The BSD 3-Clause License
- * @since     File available since Release 1.0.0
+ * For the full copyright and license information, please view the LICENSE
+ * file that was distributed with this source code.
  */
 
 /**
  * A PHP token.
  *
  * @author    Sebastian Bergmann <sebastian@phpunit.de>
- * @copyright 2009-2014 Sebastian Bergmann <sebastian@phpunit.de>
+ * @copyright Sebastian Bergmann <sebastian@phpunit.de>
  * @license   http://www.opensource.org/licenses/BSD-3-Clause  The BSD 3-Clause License
- * @version   Release: @package_version@
  * @link      http://github.com/sebastianbergmann/php-token-stream/tree
  * @since     Class available since Release 1.0.0
  */
@@ -108,6 +74,9 @@ abstract class PHP_Token
 
 abstract class PHP_TokenWithScope extends PHP_Token
 {
+    /**
+     * @var integer
+     */
     protected $endTokenId;
 
     /**
@@ -155,27 +124,26 @@ abstract class PHP_TokenWithScope extends PHP_Token
         }
     }
 
+    /**
+     * @return integer
+     */
     public function getEndTokenId()
     {
         $block  = 0;
         $i      = $this->id;
         $tokens = $this->tokenStream->tokens();
 
-        while ($this->endTokenId === NULL && isset($tokens[$i])) {
+        while ($this->endTokenId === null && isset($tokens[$i])) {
             if ($tokens[$i] instanceof PHP_Token_OPEN_CURLY ||
                 $tokens[$i] instanceof PHP_Token_CURLY_OPEN) {
                 $block++;
-            }
-
-            else if ($tokens[$i] instanceof PHP_Token_CLOSE_CURLY) {
+            } elseif ($tokens[$i] instanceof PHP_Token_CLOSE_CURLY) {
                 $block--;
 
                 if ($block === 0) {
                     $this->endTokenId = $i;
                 }
-            }
-
-            else if (($this instanceof PHP_Token_FUNCTION ||
+            } elseif (($this instanceof PHP_Token_FUNCTION ||
                 $this instanceof PHP_Token_NAMESPACE) &&
                 $tokens[$i] instanceof PHP_Token_SEMICOLON) {
                 if ($block === 0) {
@@ -186,22 +154,27 @@ abstract class PHP_TokenWithScope extends PHP_Token
             $i++;
         }
 
-        if ($this->endTokenId === NULL) {
+        if ($this->endTokenId === null) {
             $this->endTokenId = $this->id;
         }
 
         return $this->endTokenId;
     }
 
+    /**
+     * @return integer
+     */
     public function getEndLine()
     {
         return $this->tokenStream[$this->getEndTokenId()]->getLine();
     }
-
 }
 
-abstract class PHP_TokenWithScopeAndVisibility extends PHP_TokenWithScope {
-
+abstract class PHP_TokenWithScopeAndVisibility extends PHP_TokenWithScope
+{
+    /**
+     * @return string
+     */
     public function getVisibility()
     {
         $tokens = $this->tokenStream->tokens();
@@ -212,7 +185,7 @@ abstract class PHP_TokenWithScopeAndVisibility extends PHP_TokenWithScope {
                 $tokens[$i] instanceof PHP_Token_PROTECTED ||
                 $tokens[$i] instanceof PHP_Token_PUBLIC)) {
                 return strtolower(
-                  str_replace('PHP_Token_', '', get_class($tokens[$i]))
+                    str_replace('PHP_Token_', '', get_class($tokens[$i]))
                 );
             }
             if (isset($tokens[$i]) &&
@@ -225,10 +198,13 @@ abstract class PHP_TokenWithScopeAndVisibility extends PHP_TokenWithScope {
         }
     }
 
+    /**
+     * @return string
+     */
     public function getKeywords()
     {
         $keywords = array();
-        $tokens = $this->tokenStream->tokens();
+        $tokens   = $this->tokenStream->tokens();
 
         for ($i = $this->id - 2; $i > $this->id - 7; $i -= 2) {
             if (isset($tokens[$i]) &&
@@ -243,78 +219,113 @@ abstract class PHP_TokenWithScopeAndVisibility extends PHP_TokenWithScope {
                 $tokens[$i] instanceof PHP_Token_FINAL ||
                 $tokens[$i] instanceof PHP_Token_ABSTRACT)) {
                 $keywords[] = strtolower(
-                  str_replace('PHP_Token_', '', get_class($tokens[$i]))
+                    str_replace('PHP_Token_', '', get_class($tokens[$i]))
                 );
             }
         }
 
         return implode(',', $keywords);
     }
-
 }
 
 abstract class PHP_Token_Includes extends PHP_Token
 {
+    /**
+     * @var string
+     */
     protected $name;
+
+    /**
+     * @var string
+     */
     protected $type;
 
+    /**
+     * @return string
+     */
     public function getName()
     {
-        if ($this->name !== NULL) {
-            return $this->name;
-        }
-
-        $tokens = $this->tokenStream->tokens();
-
-        if ($tokens[$this->id+2] instanceof PHP_Token_CONSTANT_ENCAPSED_STRING) {
-            $this->name = trim($tokens[$this->id+2], "'\"");
-            $this->type = strtolower(
-              str_replace('PHP_Token_', '', get_class($tokens[$this->id]))
-            );
+        if ($this->name === null) {
+            $this->process();
         }
 
         return $this->name;
     }
 
+    /**
+     * @return string
+     */
     public function getType()
     {
-        $this->getName();
+        if ($this->type === null) {
+            $this->process();
+        }
+
         return $this->type;
     }
+
+    private function process()
+    {
+        $tokens = $this->tokenStream->tokens();
+
+        if ($tokens[$this->id+2] instanceof PHP_Token_CONSTANT_ENCAPSED_STRING) {
+            $this->name = trim($tokens[$this->id+2], "'\"");
+            $this->type = strtolower(
+                str_replace('PHP_Token_', '', get_class($tokens[$this->id]))
+            );
+        }
+    }
 }
 
 
 class PHP_Token_FUNCTION extends PHP_TokenWithScopeAndVisibility
 {
+    /**
+     * @var array
+     */
     protected $arguments;
+
+    /**
+     * @var integer
+     */
     protected $ccn;
+
+    /**
+     * @var string
+     */
     protected $name;
+
+    /**
+     * @var string
+     */
     protected $signature;
 
+    /**
+     * @return array
+     */
     public function getArguments()
     {
-        if ($this->arguments !== NULL) {
+        if ($this->arguments !== null) {
             return $this->arguments;
         }
 
         $this->arguments = array();
         $tokens          = $this->tokenStream->tokens();
-        $typeHint        = NULL;
+        $typeDeclaration = null;
 
         // Search for first token inside brackets
         $i = $this->id + 2;
+
         while (!$tokens[$i-1] instanceof PHP_Token_OPEN_BRACKET) {
             $i++;
         }
 
         while (!$tokens[$i] instanceof PHP_Token_CLOSE_BRACKET) {
             if ($tokens[$i] instanceof PHP_Token_STRING) {
-                $typeHint = (string)$tokens[$i];
-            }
-
-            else if ($tokens[$i] instanceof PHP_Token_VARIABLE) {
-                $this->arguments[(string)$tokens[$i]] = $typeHint;
-                $typeHint                             = NULL;
+                $typeDeclaration = (string)$tokens[$i];
+            } elseif ($tokens[$i] instanceof PHP_Token_VARIABLE) {
+                $this->arguments[(string)$tokens[$i]] = $typeDeclaration;
+                $typeDeclaration                      = null;
             }
 
             $i++;
@@ -323,9 +334,12 @@ class PHP_Token_FUNCTION extends PHP_TokenWithScopeAndVisibility
         return $this->arguments;
     }
 
+    /**
+     * @return string
+     */
     public function getName()
     {
-        if ($this->name !== NULL) {
+        if ($this->name !== null) {
             return $this->name;
         }
 
@@ -335,15 +349,11 @@ class PHP_Token_FUNCTION extends PHP_TokenWithScopeAndVisibility
             if ($tokens[$i] instanceof PHP_Token_STRING) {
                 $this->name = (string)$tokens[$i];
                 break;
-            }
-
-            else if ($tokens[$i] instanceof PHP_Token_AMPERSAND &&
+            } elseif ($tokens[$i] instanceof PHP_Token_AMPERSAND &&
                      $tokens[$i+1] instanceof PHP_Token_STRING) {
                 $this->name = (string)$tokens[$i+1];
                 break;
-            }
-
-            else if ($tokens[$i] instanceof PHP_Token_OPEN_BRACKET) {
+            } elseif ($tokens[$i] instanceof PHP_Token_OPEN_BRACKET) {
                 $this->name = 'anonymous function';
                 break;
             }
@@ -365,9 +375,12 @@ class PHP_Token_FUNCTION extends PHP_TokenWithScopeAndVisibility
         return $this->name;
     }
 
+    /**
+     * @return integer
+     */
     public function getCCN()
     {
-        if ($this->ccn !== NULL) {
+        if ($this->ccn !== null) {
             return $this->ccn;
         }
 
@@ -388,19 +401,21 @@ class PHP_Token_FUNCTION extends PHP_TokenWithScopeAndVisibility
                 case 'PHP_Token_LOGICAL_AND':
                 case 'PHP_Token_BOOLEAN_OR':
                 case 'PHP_Token_LOGICAL_OR':
-                case 'PHP_Token_QUESTION_MARK': {
+                case 'PHP_Token_QUESTION_MARK':
                     $this->ccn++;
-                }
-                break;
+                    break;
             }
         }
 
         return $this->ccn;
     }
 
+    /**
+     * @return string
+     */
     public function getSignature()
     {
-        if ($this->signature !== NULL) {
+        if ($this->signature !== null) {
             return $this->signature;
         }
 
@@ -428,29 +443,41 @@ class PHP_Token_FUNCTION extends PHP_TokenWithScopeAndVisibility
 
 class PHP_Token_INTERFACE extends PHP_TokenWithScopeAndVisibility
 {
+    /**
+     * @var array
+     */
     protected $interfaces;
 
+    /**
+     * @return string
+     */
     public function getName()
     {
         return (string)$this->tokenStream[$this->id + 2];
     }
 
+    /**
+     * @return boolean
+     */
     public function hasParent()
     {
         return $this->tokenStream[$this->id + 4] instanceof PHP_Token_EXTENDS;
     }
 
+    /**
+     * @return array
+     */
     public function getPackage()
     {
         $className  = $this->getName();
         $docComment = $this->getDocblock();
 
         $result = array(
-          'namespace'   => '',
-          'fullPackage' => '',
-          'category'    => '',
-          'package'     => '',
-          'subpackage'  => ''
+            'namespace'   => '',
+            'fullPackage' => '',
+            'category'    => '',
+            'package'     => '',
+            'subpackage'  => ''
         );
 
         for ($i = $this->id; $i; --$i) {
@@ -476,13 +503,19 @@ class PHP_Token_INTERFACE extends PHP_TokenWithScopeAndVisibility
 
         if (empty($result['fullPackage'])) {
             $result['fullPackage'] = $this->arrayToName(
-              explode('_', str_replace('\\', '_', $className)), '.'
+                explode('_', str_replace('\\', '_', $className)),
+                '.'
             );
         }
 
         return $result;
     }
 
+    /**
+     * @param  array  $parts
+     * @param  string $join
+     * @return string
+     */
     protected function arrayToName(array $parts, $join = '\\')
     {
         $result = '';
@@ -496,10 +529,13 @@ class PHP_Token_INTERFACE extends PHP_TokenWithScopeAndVisibility
         return $result;
     }
 
+    /**
+     * @return boolean|string
+     */
     public function getParent()
     {
         if (!$this->hasParent()) {
-            return FALSE;
+            return false;
         }
 
         $i         = $this->id + 6;
@@ -514,6 +550,9 @@ class PHP_Token_INTERFACE extends PHP_TokenWithScopeAndVisibility
         return $className;
     }
 
+    /**
+     * @return boolean
+     */
     public function hasInterfaces()
     {
         return (isset($this->tokenStream[$this->id + 4]) &&
@@ -522,14 +561,17 @@ class PHP_Token_INTERFACE extends PHP_TokenWithScopeAndVisibility
                 $this->tokenStream[$this->id + 8] instanceof PHP_Token_IMPLEMENTS);
     }
 
+    /**
+     * @return array|boolean
+     */
     public function getInterfaces()
     {
-        if ($this->interfaces !== NULL) {
+        if ($this->interfaces !== null) {
             return $this->interfaces;
         }
 
         if (!$this->hasInterfaces()) {
-            return ($this->interfaces = FALSE);
+            return ($this->interfaces = false);
         }
 
         if ($this->tokenStream[$this->id + 4] instanceof PHP_Token_IMPLEMENTS) {
@@ -707,12 +749,15 @@ class PHP_Token_GOTO extends PHP_Token {}
 
 class PHP_Token_NAMESPACE extends PHP_TokenWithScope
 {
+    /**
+     * @return string
+     */
     public function getName()
     {
         $tokens    = $this->tokenStream->tokens();
         $namespace = (string)$tokens[$this->id+2];
 
-        for ($i = $this->id + 3; ; $i += 2) {
+        for ($i = $this->id + 3;; $i += 2) {
             if (isset($tokens[$i]) &&
                 $tokens[$i] instanceof PHP_Token_NS_SEPARATOR) {
                 $namespace .= '\\' . $tokens[$i+1];
@@ -742,3 +787,31 @@ class PHP_Token_YIELD extends PHP_Token {}
 class PHP_Token_ELLIPSIS extends PHP_Token {}
 class PHP_Token_POW extends PHP_Token {}
 class PHP_Token_POW_EQUAL extends PHP_Token {}
+
+// Tokens introduced in PHP 7.0
+class PHP_Token_COALESCE extends PHP_Token {}
+class PHP_Token_SPACESHIP extends PHP_Token {}
+class PHP_Token_YIELD_FROM extends PHP_Token {}
+
+// Tokens introduced in HackLang / HHVM
+class PHP_Token_COMPILER_HALT_OFFSET extends PHP_Token {}
+class PHP_Token_ENUM extends PHP_Token {}
+class PHP_Token_IN extends PHP_Token {}
+class PHP_Token_JOIN extends PHP_Token {}
+class PHP_Token_LAMBDA_ARROW extends PHP_Token {}
+class PHP_Token_LAMBDA_CP extends PHP_Token {}
+class PHP_Token_LAMBDA_OP extends PHP_Token {}
+class PHP_Token_ONUMBER extends PHP_Token {}
+class PHP_Token_SHAPE extends PHP_Token {}
+class PHP_Token_TYPE extends PHP_Token {}
+class PHP_Token_TYPELIST_GT extends PHP_Token {}
+class PHP_Token_TYPELIST_LT extends PHP_Token {}
+class PHP_Token_XHP_ATTRIBUTE extends PHP_Token {}
+class PHP_Token_XHP_CATEGORY extends PHP_Token {}
+class PHP_Token_XHP_CATEGORY_LABEL extends PHP_Token {}
+class PHP_Token_XHP_CHILDREN extends PHP_Token {}
+class PHP_Token_XHP_LABEL extends PHP_Token {}
+class PHP_Token_XHP_REQUIRED extends PHP_Token {}
+class PHP_Token_XHP_TAG_GT extends PHP_Token {}
+class PHP_Token_XHP_TAG_LT extends PHP_Token {}
+class PHP_Token_XHP_TEXT extends PHP_Token {}
